@@ -69,7 +69,7 @@ class Operations(llfuse.Operations):
     explicitly checks the st_mode attribute.
     """
         
-    def __init__(self, block_cache, db, blocksize, inode_cache,
+    def __init__(self, block_cache, db, max_obj_size, inode_cache,
                  upload_event=None):
         super(Operations, self).__init__()
 
@@ -77,13 +77,16 @@ class Operations(llfuse.Operations):
         self.db = db
         self.upload_event = upload_event
         self.open_inodes = collections.defaultdict(lambda: 0)
-        self.blocksize = blocksize
+        self.max_obj_size = max_obj_size
         self.cache = block_cache
 
     def destroy(self):
+        self.forget(self.open_inodes.items())
         self.inodes.destroy()
 
     def lookup(self, id_p, name):
+        log.debug('lookup(%d, %r): start', id_p, name)
+        
         if name == CTRL_NAME:
             inode = self.inodes[CTRL_INODE]
             
@@ -91,25 +94,28 @@ class Operations(llfuse.Operations):
             # who mounted the file system (but don't mark inode as dirty)
             object.__setattr__(inode, 'uid', os.getuid())
             object.__setattr__(inode, 'gid', os.getgid())
-            
-            return inode
-            
-        if name == '.':
-            return self.inodes[id_p]
+        
+        elif name == '.':
+            inode = self.inodes[id_p]
 
-        if name == '..':
+        elif name == '..':
             id_ = self.db.get_val("SELECT parent_inode FROM contents WHERE inode=?",
                                   (id_p,))
-            return self.inodes[id_]
-
-        try:
-            id_ = self.db.get_val("SELECT inode FROM contents_v WHERE name=? AND parent_inode=?",
-                                  (name, id_p))
-        except NoSuchRowError:
-            raise(llfuse.FUSEError(errno.ENOENT))
-        return self.inodes[id_]
+            inode = self.inodes[id_]
+            
+        else:
+            try:
+                id_ = self.db.get_val("SELECT inode FROM contents_v WHERE name=? AND parent_inode=?",
+                                      (name, id_p))
+            except NoSuchRowError:
+                raise(llfuse.FUSEError(errno.ENOENT))
+            inode = self.inodes[id_]
+            
+        self.open_inodes[inode.id] += 1
+        return inode
 
     def getattr(self, id_):
+        log.debug('getattr(%d): start', id_)
         if id_ == CTRL_INODE:
             # Make sure the control file is only writable by the user
             # who mounted the file system (but don't mark inode as dirty)
@@ -118,14 +124,10 @@ class Operations(llfuse.Operations):
             object.__setattr__(inode, 'gid', os.getgid())
             return inode
             
-        try:
-            return self.inodes[id_]
-        except KeyError:
-            # It is possible to get getattr() for an inode that
-            # has just been unlinked()
-            raise FUSEError(errno.ENOENT)
+        return self.inodes[id_]
 
     def readlink(self, id_):
+        log.debug('readlink(%d): start', id_)
         timestamp = time.time()
         inode = self.inodes[id_]
         if inode.atime < inode.ctime or inode.atime < inode.mtime:
@@ -137,6 +139,7 @@ class Operations(llfuse.Operations):
             raise FUSEError(errno.EINVAL)
 
     def opendir(self, id_):
+        log.debug('opendir(%d): start', id_)
         return id_
 
     def check_args(self, args):
@@ -147,6 +150,8 @@ class Operations(llfuse.Operations):
         args.append('no_remote_lock')
 
     def readdir(self, id_, off):
+        # FIXME: Do the returned entries acquire a lookup count?
+        log.debug('readdir(%d, %d): start', id_, off)
         if off == 0:
             off = -1
             
@@ -156,12 +161,13 @@ class Operations(llfuse.Operations):
 
         # The ResultSet is automatically deleted
         # when yield raises GeneratorExit.  
-        res = self.db.query("SELECT rowid, name, inode FROM contents_v "
-                            'WHERE parent_inode=? AND contents_v.rowid > ? ORDER BY rowid', (id_, off))
+        res = self.db.query("SELECT name_id, name, inode FROM contents_v "
+                            'WHERE parent_inode=? AND name_id > ? ORDER BY name_id', (id_, off))
         for (next_, name, cid_) in res:
             yield (name, self.inodes[cid_], next_)
 
     def getxattr(self, id_, name):
+        log.debug('getxattr(%d, %r): start', id_, name)
         # Handle S3QL commands
         if id_ == CTRL_INODE:
             if name == b's3ql_pid?':
@@ -181,13 +187,15 @@ class Operations(llfuse.Operations):
             return value
 
     def listxattr(self, id_):
+        log.debug('listxattr(%d): start', id_)
         names = list()
         for (name,) in self.db.query('SELECT name FROM ext_attributes_v WHERE inode=?', (id_,)):
             names.append(name)
         return names
 
     def setxattr(self, id_, name, value):
-        
+        log.debug('setxattr(%d, %r, %r): start', id_,name, value)
+                  
         # Handle S3QL commands
         if id_ == CTRL_INODE:
             if name == b's3ql_flushcache!':
@@ -212,7 +220,7 @@ class Operations(llfuse.Operations):
         else:
             if self.inodes[id_].locked:
                 raise FUSEError(errno.EPERM)
-                    
+                      
             if len(value) > deltadump.MAX_BLOB_SIZE:
                 raise FUSEError(errno.EINVAL)
             
@@ -221,10 +229,11 @@ class Operations(llfuse.Operations):
             self.inodes[id_].ctime = time.time()
 
     def removexattr(self, id_, name):
+        log.debug('removexattr(%d, %r): start', id_, name)
         
         if self.inodes[id_].locked:
             raise FUSEError(errno.EPERM)
-            
+               
         try:
             name_id =  self._del_name(name)
         except NoSuchRowError:
@@ -245,7 +254,7 @@ class Operations(llfuse.Operations):
         self.inodes[id0].locked = True
         processed = 0 # Number of steps since last GIL release
         stamp = time.time() # Time of last GIL release
-        gil_step = 500 # Approx. number of steps between GIL releases
+        gil_step = 250 # Approx. number of steps between GIL releases
         while True:    
             id_p = queue.pop()
             for (id_,) in self.db.query('SELECT inode FROM contents WHERE parent_inode=?',
@@ -261,11 +270,12 @@ class Operations(llfuse.Operations):
                             
             if processed > gil_step:
                 dt = time.time() - stamp
-                gil_step = max(int(gil_step * GIL_RELEASE_INTERVAL / dt), 1)
+                gil_step = max(int(gil_step * GIL_RELEASE_INTERVAL / dt), 250)
                 log.debug('lock_tree(%d): Adjusting gil_step to %d', 
                           id0, gil_step)  
                 processed = 0
-                llfuse.lock.yield_()
+                llfuse.lock.yield_(100)
+                log.debug('lock_tree(%d): re-acquired lock', id0)
                 stamp = time.time()
 
         log.debug('lock_tree(%d): end', id0)
@@ -282,7 +292,7 @@ class Operations(llfuse.Operations):
         queue = [ id0 ]
         processed = 0 # Number of steps since last GIL release
         stamp = time.time() # Time of last GIL release
-        gil_step = 50 # Approx. number of steps between GIL releases
+        gil_step = 250 # Approx. number of steps between GIL releases
         while True:
             found_subdirs = False
             id_p = queue.pop()  
@@ -297,7 +307,8 @@ class Operations(llfuse.Operations):
                     
                 else:
                     name = self.db.get_val("SELECT name FROM names WHERE id=?", (name_id,))
-                    llfuse.invalidate_entry(id_p, name)
+                    if id_p in self.open_inodes:
+                        llfuse.invalidate_entry(id_p, name)
                     self._remove(id_p, name, id_, force=True)
                 
                 processed += 1   
@@ -308,19 +319,22 @@ class Operations(llfuse.Operations):
                     break
                 
             if not queue:
-                llfuse.invalidate_entry(id_p0, name0)
+                if id_p0 in self.open_inodes:
+                    llfuse.invalidate_entry(id_p0, name0)
                 self._remove(id_p0, name0, id0, force=True)
                 break              
           
             if processed > gil_step:  
                 dt = time.time() - stamp
-                gil_step = max(int(gil_step * GIL_RELEASE_INTERVAL / dt), 1)
-                log.debug('remove_tree(%d, %s): Adjusting gil_step to %d', 
+                gil_step = max(int(gil_step * GIL_RELEASE_INTERVAL / dt), 250)
+                log.debug('remove_tree(%d, %s): Adjusting gil_step to %d and yielding', 
                           id_p0, name0, gil_step)  
                 processed = 0
-                llfuse.lock.yield_()
+                llfuse.lock.yield_(100)
+                log.debug('remove_tree(%d, %s): re-acquired lock', id_p0, name0)
                 stamp = time.time()    
         
+        self.forget([(id0, 1)])
         log.debug('remove_tree(%d, %s): end', id_p0, name0)
         
     
@@ -338,8 +352,13 @@ class Operations(llfuse.Operations):
         log.debug('copy_tree(%d, %d): committed cache', src_id, target_id)
 
         # Copy target attributes
-        src_inode = self.inodes[src_id]
-        target_inode = self.inodes[target_id]
+        # These come from setxattr, so they may have been deleted
+        # without being in open_inodes
+        try:
+            src_inode = self.inodes[src_id]
+            target_inode = self.inodes[target_id]
+        except KeyError:
+            raise FUSEError(errno.ENOENT)            
         for attr in ('atime', 'ctime', 'mtime', 'mode', 'uid', 'gid'):
             setattr(target_inode, attr, getattr(src_inode, attr))
 
@@ -353,14 +372,14 @@ class Operations(llfuse.Operations):
         id_cache = dict()
         processed = 0 # Number of steps since last GIL release
         stamp = time.time() # Time of last GIL release
-        gil_step = 100 # Approx. number of steps between GIL releases
+        gil_step = 250 # Approx. number of steps between GIL releases
         while queue:
-            (src_id, target_id, rowid) = queue.pop()
+            (src_id, target_id, off) = queue.pop()
             log.debug('copy_tree(%d, %d): Processing directory (%d, %d, %d)', 
-                      src_inode.id, target_inode.id, src_id, target_id, rowid)
-            for (name_id, id_, rowid) in db.query('SELECT name_id, inode, rowid FROM contents '
-                                                  'WHERE parent_inode=? AND rowid > ? '
-                                                  'ORDER BY rowid', (src_id, rowid)):
+                      src_inode.id, target_inode.id, src_id, target_id, off)
+            for (name_id, id_) in db.query('SELECT name_id, inode FROM contents '
+                                           'WHERE parent_inode=? AND name_id > ? '
+                                           'ORDER BY name_id', (src_id, off)):
 
                 if id_ not in id_cache:
                     inode = self.inodes[id_]
@@ -412,17 +431,19 @@ class Operations(llfuse.Operations):
                 
                 if processed > gil_step:
                     log.debug('copy_tree(%d, %d): Requeueing (%d, %d, %d) to yield lock', 
-                              src_inode.id, target_inode.id, src_id, target_id, rowid)
-                    queue.append((src_id, target_id, rowid))
+                              src_inode.id, target_inode.id, src_id, target_id, name_id)
+                    queue.append((src_id, target_id, name_id))
                     break
             
             if processed > gil_step:
                 dt = time.time() - stamp
-                gil_step = max(int(gil_step * GIL_RELEASE_INTERVAL / dt), 1)
-                log.debug('copy_tree(%d, %d): Adjusting gil_step to %d', 
+                gil_step = max(int(gil_step * GIL_RELEASE_INTERVAL / dt), 250)
+                log.debug('copy_tree(%d, %d): Adjusting gil_step to %d and yielding', 
                           src_inode.id, target_inode.id, gil_step) 
                 processed = 0
-                llfuse.lock.yield_()
+                llfuse.lock.yield_(100)
+                log.debug('copy_tree(%d, %d): re-acquired lock',
+                          src_inode.id, target_inode.id)
                 stamp = time.time()
 
         # Make replication visible
@@ -434,14 +455,18 @@ class Operations(llfuse.Operations):
         log.debug('copy_tree(%d, %d): end', src_inode.id, target_inode.id)
 
     def unlink(self, id_p, name):
+        log.debug('rmdir(%d, %r): start', id_p, name)
         inode = self.lookup(id_p, name)
 
         if stat.S_ISDIR(inode.mode):
             raise llfuse.FUSEError(errno.EISDIR)
 
         self._remove(id_p, name, inode.id)
-
+        
+        self.forget([(inode.id, 1)])
+        
     def rmdir(self, id_p, name):
+        log.debug('rmdir(%d, %r): start', id_p, name)
         inode = self.lookup(id_p, name)
 
         if self.inodes[id_p].locked:
@@ -452,6 +477,7 @@ class Operations(llfuse.Operations):
 
         self._remove(id_p, name, inode.id)
 
+        self.forget([(inode.id, 1)])
 
     def _remove(self, id_p, name, id_, force=False):
         '''Remove entry `name` with parent inode `id_p` 
@@ -489,13 +515,15 @@ class Operations(llfuse.Operations):
 
         if inode.refcount == 0 and id_ not in self.open_inodes:
             log.debug('_remove(%d, %s): removing from cache', id_p, name)
-            self.cache.remove(id_, 0, int(math.ceil(inode.size / self.blocksize)))
+            self.cache.remove(id_, 0, int(math.ceil(inode.size / self.max_obj_size)))
             # Since the inode is not open, it's not possible that new blocks
             # get created at this point and we can safely delete the inode
             self.db.execute('UPDATE names SET refcount = refcount - 1 WHERE '
                             'id IN (SELECT name_id FROM ext_attributes WHERE inode=?)', 
                             (id_,))
-            self.db.execute('DELETE FROM names WHERE refcount=0')                 
+            self.db.execute('DELETE FROM names WHERE refcount=0 AND '
+                            'id IN (SELECT name_id FROM ext_attributes WHERE inode=?)', 
+                            (id_,))                                       
             self.db.execute('DELETE FROM ext_attributes WHERE inode=?', (id_,))
             self.db.execute('DELETE FROM symlink_targets WHERE inode=?', (id_,))
             del self.inodes[id_]
@@ -503,6 +531,8 @@ class Operations(llfuse.Operations):
         log.debug('_remove(%d, %s): start', id_p, name)
 
     def symlink(self, id_p, name, target, ctx):
+        log.debug('symlink(%d, %r, %r): start', id_p, name, target)
+        
         mode = (stat.S_IFLNK | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | 
                 stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP | 
                 stat.S_IROTH | stat.S_IWOTH | stat.S_IXOTH)
@@ -515,10 +545,11 @@ class Operations(llfuse.Operations):
         inode = self._create(id_p, name, mode, ctx, size=len(target))
         self.db.execute('INSERT INTO symlink_targets (inode, target) VALUES(?,?)',
                         (inode.id, target))
-        
+        self.open_inodes[inode.id] += 1
         return inode
 
     def rename(self, id_p_old, name_old, id_p_new, name_new):
+        log.debug('rename(%d, %r, %d, %r): start', id_p_old, name_old, id_p_new, name_new)
         if name_new == CTRL_NAME or name_old == CTRL_NAME:
             log.warn('Attempted to rename s3ql control file (%s -> %s)',
                       get_path(id_p_old, self.db, name_old),
@@ -541,14 +572,14 @@ class Operations(llfuse.Operations):
         else:
             target_exists = True
 
-
         if target_exists:
             self._replace(id_p_old, name_old, id_p_new, name_new,
                           inode_old.id, inode_new.id)
+            self.forget([(inode_old.id, 1), (inode_new.id, 1)])
         else:
             self._rename(id_p_old, name_old, id_p_new, name_new)
-
-
+            self.forget([(inode_old.id, 1)])
+        
     def _add_name(self, name):
         '''Get id for *name* and increase refcount
         
@@ -612,7 +643,6 @@ class Operations(llfuse.Operations):
         self.db.execute("UPDATE contents SET inode=? WHERE name_id=? AND parent_inode=?",
                         (id_old, name_id_new, id_p_new))
         
-
         # Delete old name
         name_id_old = self._del_name(name_old)          
         self.db.execute('DELETE FROM contents WHERE name_id=? AND parent_inode=?',
@@ -632,7 +662,7 @@ class Operations(llfuse.Operations):
 
         if inode_new.refcount == 0 and id_new not in self.open_inodes:
             self.cache.remove(id_new, 0, 
-                              int(math.ceil(inode_new.size / self.blocksize)))
+                              int(math.ceil(inode_new.size / self.max_obj_size)))
             # Since the inode is not open, it's not possible that new blocks
             # get created at this point and we can safely delete the inode
             self.db.execute('UPDATE names SET refcount = refcount - 1 WHERE '
@@ -645,6 +675,8 @@ class Operations(llfuse.Operations):
 
 
     def link(self, id_, new_id_p, new_name):
+        log.debug('link(%d, %d, %r): start', id_, new_id_p, new_name)
+        
         if new_name == CTRL_NAME or id_ == CTRL_INODE:
             log.warn('Attempted to create s3ql control file at %s',
                       get_path(new_id_p, self.db, new_name))
@@ -669,13 +701,18 @@ class Operations(llfuse.Operations):
         inode = self.inodes[id_]
         inode.refcount += 1
         inode.ctime = timestamp
-
+        
+        self.open_inodes[inode.id] += 1
         return inode
 
     def setattr(self, id_, attr):
         """Handles FUSE setattr() requests"""
-
-        inode = self.inodes[id_]
+        if log.isEnabledFor(logging.DEBUG):
+            log.debug('setattr(%d, %s): start', id_,
+                      [ getattr(attr, x) for x in attr.__slots__ 
+                       if getattr(attr, x) is not None ])
+        
+        inode = self.inodes[id_]       
         timestamp = time.time()
 
         if inode.locked:
@@ -685,9 +722,9 @@ class Operations(llfuse.Operations):
             len_ = attr.st_size
 
             # Determine blocks to delete
-            last_block = len_ // self.blocksize
-            cutoff = len_ % self.blocksize
-            total_blocks = int(math.ceil(inode.size / self.blocksize)) 
+            last_block = len_ // self.max_obj_size
+            cutoff = len_ % self.max_obj_size
+            total_blocks = int(math.ceil(inode.size / self.max_obj_size)) 
             
             # Adjust file size
             inode.size = len_
@@ -697,21 +734,18 @@ class Operations(llfuse.Operations):
                 self.cache.remove(id_, last_block, total_blocks)
             else:
                 self.cache.remove(id_, last_block + 1, total_blocks)
-                
                 try:
                     with self.cache.get(id_, last_block) as fh:
                         fh.truncate(cutoff)
-                        
                 except NoSuchObject as exc:
                     log.warn('Backend lost block %d of inode %d (id %s)!', 
                              last_block, id_, exc.key)
-                    raise FUSEError(errno.EIO)
-                
+                    raise 
+                                    
                 except ChecksumError as exc:
                     log.warn('Backend returned malformed data for block %d of inode %d (%s)',
                              last_block, id_, exc)
-                    raise FUSEError(errno.EIO)
-            
+                    raise                          
 
         if attr.st_mode is not None:
             inode.mode = attr.st_mode
@@ -739,14 +773,22 @@ class Operations(llfuse.Operations):
         return inode
 
     def mknod(self, id_p, name, mode, rdev, ctx):
-        return self._create(id_p, name, mode, ctx, rdev=rdev)
+        log.debug('mknod(%d, %r): start', id_p, name)
+        inode = self._create(id_p, name, mode, ctx, rdev=rdev)
+        self.open_inodes[inode.id] += 1
+        return inode
 
     def mkdir(self, id_p, name, mode, ctx):
-        return self._create(id_p, name, mode, ctx)
+        log.debug('mkdir(%d, %r): start', id_p, name)
+        inode = self._create(id_p, name, mode, ctx)
+        self.open_inodes[inode.id] += 1
+        return inode
 
     def extstat(self):
         '''Return extended file system statistics'''
 
+        log.debug('extstat(%d): start')
+        
         # Flush inode cache to get better estimate of total fs size
         self.inodes.flush()
         
@@ -762,6 +804,8 @@ class Operations(llfuse.Operations):
 
 
     def statfs(self):
+        log.debug('statfs(): start')
+        
         stat_ = llfuse.StatvfsData
 
         # Get number of blocks & inodes
@@ -776,8 +820,8 @@ class Operations(llfuse.Operations):
         # It would be more appropriate to switch f_bsize and f_frsize,
         # but since df and stat ignore f_frsize, this way we can
         # export more information  
-        stat_.f_bsize = int(size // blocks) if blocks != 0 else self.blocksize
-        stat_.f_frsize = self.blocksize
+        stat_.f_bsize = int(size // blocks) if blocks != 0 else self.max_obj_size
+        stat_.f_frsize = self.max_obj_size
 
         # size of fs in f_frsize units 
         # (since backend is supposed to be unlimited, always return a half-full filesystem,
@@ -796,11 +840,11 @@ class Operations(llfuse.Operations):
         return stat_
 
     def open(self, id_, flags):
-        if (self.inodes[id_].locked and
-            (flags & os.O_RDWR or flags & os.O_WRONLY)):
+        log.debug('open(%d): start', id_)
+        if ((flags & os.O_RDWR or flags & os.O_WRONLY)
+            and self.inodes[id_].locked):
             raise FUSEError(errno.EPERM)
         
-        self.open_inodes[id_] += 1
         return id_
 
     def access(self, id_, mode, ctx):
@@ -813,10 +857,20 @@ class Operations(llfuse.Operations):
         # Yeah, could be a function and has unused arguments
         #pylint: disable=R0201,W0613
 
+        log.debug('access(%d): executed', id_)
         return True
 
-    def create(self, id_p, name, mode, ctx):
-        inode = self._create(id_p, name, mode, ctx)
+    def create(self, id_p, name, mode, flags, ctx):
+        log.debug('create(id_p=%d, %s): started', id_p, name)
+        try:
+            id_ = self.db.get_val("SELECT inode FROM contents_v WHERE name=? AND parent_inode=?",
+                                  (name, id_p))
+        except NoSuchRowError:
+            inode = self._create(id_p, name, mode, ctx)
+        else:
+            self.open(id_, flags)
+            inode = self.inodes[id_]
+        
         self.open_inodes[inode.id] += 1
         return (inode.id, inode)
 
@@ -860,6 +914,7 @@ class Operations(llfuse.Operations):
         
         This method releases the global lock while it is running.
         '''
+        log.debug('read(%d, %d, %d): start', fh, offset, length)
         buf = StringIO()
         inode = self.inodes[fh]
 
@@ -886,7 +941,7 @@ class Operations(llfuse.Operations):
     def _read(self, id_, offset, length):
         """Reads at the specified position until the end of the block
 
-        This method may return less than `length` bytes if a blocksize
+        This method may return less than `length` bytes if a max_obj_size
         boundary is encountered. It may also read beyond the end of
         the file, filling the buffer with additional null bytes.
         
@@ -894,12 +949,12 @@ class Operations(llfuse.Operations):
         """
 
         # Calculate required block
-        blockno = offset // self.blocksize
-        offset_rel = offset - blockno * self.blocksize
+        blockno = offset // self.max_obj_size
+        offset_rel = offset - blockno * self.max_obj_size
 
         # Don't try to read into the next block
-        if offset_rel + length > self.blocksize:
-            length = self.blocksize - offset_rel
+        if offset_rel + length > self.max_obj_size:
+            length = self.max_obj_size - offset_rel
 
         try:
             with self.cache.get(id_, blockno) as fh:
@@ -909,12 +964,12 @@ class Operations(llfuse.Operations):
         except NoSuchObject as exc:
             log.warn('Backend lost block %d of inode %d (id %s)!', 
                      blockno, id_, exc.key)
-            raise FUSEError(errno.EIO)
+            raise
         
         except ChecksumError as exc:
             log.warn('Backend returned malformed data for block %d of inode %d (%s)',
                      blockno, id_, exc)
-            raise FUSEError(errno.EIO)
+            raise 
 
         if len(buf) == length:
             return buf
@@ -927,6 +982,7 @@ class Operations(llfuse.Operations):
         
         This method releases the global lock while it is running.
         '''
+        log.debug('write(%d, %d, datalen=%d): start', fh, offset, len(buf))
         
         if self.inodes[fh].locked:
             raise FUSEError(errno.EPERM)
@@ -961,12 +1017,12 @@ class Operations(llfuse.Operations):
         """
 
         # Calculate required block
-        blockno = offset // self.blocksize
-        offset_rel = offset - blockno * self.blocksize
+        blockno = offset // self.max_obj_size
+        offset_rel = offset - blockno * self.max_obj_size
 
         # Don't try to write into the next block
-        if offset_rel + len(buf) > self.blocksize:
-            buf = buf[:self.blocksize - offset_rel]
+        if offset_rel + len(buf) > self.max_obj_size:
+            buf = buf[:self.max_obj_size - offset_rel]
 
         try:
             with self.cache.get(id_, blockno) as fh:
@@ -976,49 +1032,62 @@ class Operations(llfuse.Operations):
         except NoSuchObject as exc:
             log.warn('Backend lost block %d of inode %d (id %s)!', 
                      blockno, id_, exc.key)
-            raise FUSEError(errno.EIO)
+            raise 
                             
         except ChecksumError as exc:
             log.warn('Backend returned malformed data for block %d of inode %d (%s)',
                      blockno, id_, exc)
-            raise FUSEError(errno.EIO)
+            raise 
                 
         return len(buf)
 
     def fsync(self, fh, datasync):
+        log.debug('fsync(%d, %s): start', fh, datasync)
         if not datasync:
             self.inodes.flush_id(fh)
 
         self.cache.flush(fh)
+        
+    def forget(self, forget_list):
+        log.debug('forget(%s): start', forget_list)
+        
+        for (id_, nlookup) in forget_list:
+            self.open_inodes[id_] -= nlookup
 
-    def releasedir(self, fh):
-        # Unused argument
-        #pylint: disable=W0613
-        return
-
-    def release(self, fh):
-        self.open_inodes[fh] -= 1
-
-        if self.open_inodes[fh] == 0:
-            del self.open_inodes[fh]
-
-            inode = self.inodes[fh]
-            if inode.refcount == 0:
-                self.cache.remove(inode.id, 0, 
-                                  int(math.ceil(inode.size / self.blocksize)))
-                # Since the inode is not open, it's not possible that new blocks
-                # get created at this point and we can safely delete the in
-                del self.inodes[fh]
-
-
-    # Called for close() calls. 
-    def flush(self, fh):
-        pass
+            if self.open_inodes[id_] == 0:
+                del self.open_inodes[id_]
+    
+                inode = self.inodes[id_]
+                if inode.refcount == 0:
+                    log.debug('_forget(%s): removing %d from cache', forget_list, id_)
+                    self.cache.remove(id_, 0, int(math.ceil(inode.size / self.max_obj_size)))
+                    # Since the inode is not open, it's not possible that new blocks
+                    # get created at this point and we can safely delete the inode
+                    self.db.execute('UPDATE names SET refcount = refcount - 1 WHERE '
+                                    'id IN (SELECT name_id FROM ext_attributes WHERE inode=?)', 
+                                    (id_,))
+                    self.db.execute('DELETE FROM names WHERE refcount=0 AND '
+                                    'id IN (SELECT name_id FROM ext_attributes WHERE inode=?)', 
+                                    (id_,))                                       
+                    self.db.execute('DELETE FROM ext_attributes WHERE inode=?', (id_,))
+                    self.db.execute('DELETE FROM symlink_targets WHERE inode=?', (id_,))
+                    del self.inodes[id_]
+                                
 
     def fsyncdir(self, fh, datasync):
+        log.debug('fsyncdir(%d, %s): start', fh, datasync)
         if not datasync:
             self.inodes.flush_id(fh)
 
+    def releasedir(self, fh):
+        log.debug('releasedir(%d): start', fh)
+
+    def release(self, fh):
+        log.debug('release(%d): start', fh)
+        
+    def flush(self, fh):
+        log.debug('flush(%d): start', fh)
+            
 def update_logging(level, modules):           
     root_logger = logging.getLogger()
     if level == logging.DEBUG:
