@@ -45,7 +45,7 @@ class Fsck(object):
         self.expect_errors = False
         self.found_errors = False
         self.uncorrectable_errors = False
-        self.blocksize = param['blocksize']
+        self.max_obj_size = param['max_obj_size']
         self.conn = conn     
         
         # Set of blocks that have been unlinked by check_cache.
@@ -177,7 +177,7 @@ class Fsck(object):
                 (block_id, obj_id) = self.conn.get_row('SELECT id, obj_id FROM blocks WHERE hash=?', (hash_,))
                 
             except NoSuchRowError:
-                obj_id = self.conn.rowid('INSERT INTO objects (refcount) VALUES(1)')
+                obj_id = self.conn.rowid('INSERT INTO objects (refcount, size) VALUES(1, -1)')
                 block_id = self.conn.rowid('INSERT INTO blocks (refcount, hash, obj_id, size) '
                                            'VALUES(?, ?, ?, ?)', (1, hash_, obj_id, size))
                 def do_write(obj_fh):
@@ -423,7 +423,7 @@ class Fsck(object):
             INSERT INTO min_sizes (id, min_size) 
             SELECT inode, MAX(blockno * ? + size) 
             FROM inode_blocks JOIN blocks ON block_id == blocks.id 
-            GROUP BY inode''', (self.blocksize,))
+            GROUP BY inode''', (self.max_obj_size,))
             
             self.conn.execute('''
                CREATE TEMPORARY TABLE wrong_sizes AS 
@@ -892,7 +892,7 @@ class Fsck(object):
     
         log.info('Checking objects (sizes)...')
     
-        for (obj_id,) in self.conn.query('SELECT id FROM objects WHERE size IS NULL'):
+        for (obj_id,) in self.conn.query('SELECT id FROM objects WHERE size = -1 OR size IS NULL'):
             self.found_errors = True
             self.log_error("Object %d has no size information, retrieving from backend...",  obj_id)               
         
@@ -1125,7 +1125,9 @@ def main(args=None):
         param['seq_no'] = seq_no
         param['needs_fsck'] = True
     
-    
+    if 'max_obj_size' not in param:
+        param['max_obj_size'] = param['blocksize']
+            
     if (not param['needs_fsck']
         and param['max_inode'] < 2**31 
         and ((time.time() - time.timezone) - param['last_fsck'])
@@ -1188,6 +1190,10 @@ def main(args=None):
         param['inode_gen'] += 1
         param['max_inode'] = db.get_val('SELECT MAX(id) FROM inodes')
             
+    if fsck.found_errors and not param['needs_fsck']:
+        log.warn('File system was marked as clean, yet fsck found problems.')
+        log.warn('Please report this to the S3QL mailing list, http://groups.google.com/group/s3ql')
+        
     cycle_metadata(bucket)
     param['needs_fsck'] = False
     param['last_fsck'] = time.time() - time.timezone
