@@ -10,12 +10,8 @@ from __future__ import division, print_function, absolute_import
 
 import time
 import logging
-from random import randint
-import apsw
 from .database import NoSuchRowError
-from s3ql.common import CTRL_INODE
 
-__all__ = [ 'InodeCache', 'OutOfInodesError' ]
 log = logging.getLogger('inode_cache')
 
 CACHE_SIZE = 100
@@ -28,16 +24,16 @@ UPDATE_STR = ', '.join('%s=?' % x for x in UPDATE_ATTRS)
 TIMEZONE = time.timezone
 
 MAX_INODE = 2**32 - 1 
-MIN_INODE = CTRL_INODE+1
 
 class _Inode(object):
     '''An inode with its attributes'''
 
-    __slots__ = ATTRIBUTES + ('dirty',)
+    __slots__ = ATTRIBUTES + ('dirty', 'generation')
 
-    def __init__(self):
+    def __init__(self, generation):
         super(_Inode, self).__init__()
         self.dirty = False
+        self.generation = generation
 
     # This allows access to all st_* attributes, even if they're
     # not defined in the table
@@ -46,7 +42,7 @@ class _Inode(object):
             return self.refcount
 
         elif key == 'st_blocks':
-            return self.size // 512
+            return (self.size+511) // 512
 
         elif key == 'st_ino':
             return self.id
@@ -61,10 +57,6 @@ class _Inode(object):
         elif key == 'st_blksize':
             return 128 * 1024
 
-        # Our inodes are already unique
-        elif key == 'generation':
-            return 1
-
         elif key.startswith('st_'):
             return getattr(self, key[3:])
 
@@ -80,7 +72,7 @@ class _Inode(object):
 
 
     def copy(self):
-        copy = _Inode()
+        copy = _Inode(self.generation)
 
         for attr in ATTRIBUTES:
             setattr(copy, attr, getattr(self, attr))
@@ -130,10 +122,11 @@ class InodeCache(object):
     to the effects of the current method call.
     '''
 
-    def __init__(self, db):
+    def __init__(self, db, inode_gen):
         self.attrs = dict()
         self.cached_rows = list()
         self.db = db
+        self.generation = inode_gen
 
         # Fill the cache with dummy data, so that we don't have to
         # check if the cache is full or not (it will always be full)        
@@ -175,10 +168,10 @@ class InodeCache(object):
             self.attrs[id_] = inode
             return inode
 
-    def getattr(self, id_):
+    def getattr(self, id_): #@ReservedAssignment
         attrs = self.db.get_row("SELECT %s FROM inodes WHERE id=? " % ATTRIBUTE_STR,
                                   (id_,))
-        inode = _Inode()
+        inode = _Inode(self.generation)
 
         for (i, id_) in enumerate(ATTRIBUTES):
             setattr(inode, id_, attrs[i])
@@ -203,19 +196,10 @@ class InodeCache(object):
         columns = ', '.join(x for x in ATTRIBUTES if x in kw)
         values = ', '.join('?' * len(kw))
                                   
-        # We want to restrict inodes to 32bit, and we do not want to
-        # immediately reuse deleted inodes (so that the lack of generation
-        # numbers isn't too likely to cause problems with NFS)
-        sql = 'INSERT INTO inodes (id, %s) VALUES(?, %s)' % (columns, values)
-        for _ in range(100):
-            id_ = randint(MIN_INODE, MAX_INODE)
-            try:
-                self.db.execute(sql, (id_,) + bindings)
-            except apsw.ConstraintError:
-                pass
-            else:
-                break
-        else:
+        id_ = self.db.rowid('INSERT INTO inodes (%s) VALUES(%s)' % (columns, values),
+                            bindings)
+        if id_ > MAX_INODE-1:
+            self.db.execute('DELETE FROM inodes WHERE id=?', (id_,))
             raise OutOfInodesError()
 
         return self[id_]
